#include "ItemIconRichTextBlockImageDecorator.h"
#include "Fonts/FontMeasure.h"
#include "Framework/Application/SlateApplication.h"
#include "Widgets/Layout/SBox.h"
#include "Widgets/Layout/SScaleBox.h"
#include "Widgets/SCanvas.h"
#include "Widgets/Images/SImage.h"
#include "Layout/Margin.h"
#include "Misc/DefaultValueHelper.h"
#define LOCTEXT_NAMESPACE "UMG"

#pragma region Item Icon Rich Inline Image

#pragma region Slate

class SItemIconRichInlineImage : public SCompoundWidget
{
public:
	SLATE_BEGIN_ARGS(SItemIconRichInlineImage)
	{
	}
	SLATE_END_ARGS()
public:
	void Construct(const FArguments& InArgs, const FSlateBrush* Brush, const FTextBlockStyle& TextStyle, EStretch::Type Stretch, FCustomRichInlineImageArguments CustomRichInlineImageArguments)
	{
		if (ensure(Brush))
		{
			const TSharedRef<FSlateFontMeasure> FontMeasure = FSlateApplication::Get().GetRenderer()->GetFontMeasureService();
			float IconHeight = FMath::Min((float)FontMeasure->GetMaxCharacterHeight(TextStyle.Font, 1.0f), Brush->ImageSize.Y);
			float IconWidth = IconHeight;
			FMargin Margin;
			float PositionX = 0.0f;
			float PositionY = 0.0f;
			if (CustomRichInlineImageArguments.Width != -1)
			{
				IconWidth = CustomRichInlineImageArguments.Width;
			}
			if (CustomRichInlineImageArguments.Height != -1)
			{
				IconHeight = CustomRichInlineImageArguments.Height;
			}
			if (CustomRichInlineImageArguments.MarginTop != 0 || 
				CustomRichInlineImageArguments.MarginBottom != 0 || 
				CustomRichInlineImageArguments.MarginRight != 0 || 
				CustomRichInlineImageArguments.MarginLeft != 0) 
			{
				if (CustomRichInlineImageArguments.MarginTop != 0) {
					Margin.Top = CustomRichInlineImageArguments.MarginTop;
				}
				if (CustomRichInlineImageArguments.MarginBottom != 0) {
					Margin.Bottom = CustomRichInlineImageArguments.MarginBottom;
				}
				if (CustomRichInlineImageArguments.MarginRight != 0) {
					Margin.Right = CustomRichInlineImageArguments.MarginRight;
				}
				if (CustomRichInlineImageArguments.MarginLeft != 0) {
					Margin.Left = CustomRichInlineImageArguments.MarginLeft;
				}
			}
			if (CustomRichInlineImageArguments.PositionX != 0.0f) {
				PositionX = CustomRichInlineImageArguments.PositionX;
			}
			if (CustomRichInlineImageArguments.PositionY != 0.0f) {
				PositionY = CustomRichInlineImageArguments.PositionY;
			}
			ChildSlot
				[
					SNew(SBox)
					.Padding(Margin)
				.HeightOverride(IconHeight)
				.WidthOverride(IconWidth)
				[
					SNew(SScaleBox)
					.Stretch(Stretch)
				.StretchDirection(EStretchDirection::DownOnly)
				.VAlign(VAlign_Center)
				[
					SNew(SCanvas)
					+ SCanvas::Slot()
				.Position(FVector2D(PositionX, PositionY))
				.Size(FVector2D(IconWidth, IconHeight))
				.HAlign(HAlign_Center)
				.VAlign(VAlign_Center)
				[
					SNew(SImage)
					.Image(Brush)
				]
				]
				]
				];
		}
	}
};

#pragma endregion

#pragma region Struct

class FItemIconRichInlineImage : public FRichTextDecorator
{
public:
	FItemIconRichInlineImage(URichTextBlock* InOwner, UItemIconRichTextBlockImageDecorator* InDecorator)
		: FRichTextDecorator(InOwner)
		, Decorator(InDecorator)
	{
	}
	virtual bool Supports(const FTextRunParseResults& RunParseResult, const FString& Text) const override
	{
		if (RunParseResult.Name == TEXT("itemimg") && 
			(
				RunParseResult.MetaData.Contains(TEXT("itemtype")) && RunParseResult.MetaData.Contains(TEXT("itemunit"))
				))
		{
			const FTextRange& ItemTypeRange = RunParseResult.MetaData[TEXT("itemtype")];
			const FString ItemTypeTag = Text.Mid(
				ItemTypeRange.BeginIndex, ItemTypeRange.EndIndex - ItemTypeRange.BeginIndex);
			const FTextRange& ItemUnitRange = RunParseResult.MetaData[TEXT("itemunit")];
			const FString ItemUnitTag = Text.Mid(
				ItemUnitRange.BeginIndex, ItemUnitRange.EndIndex - ItemUnitRange.BeginIndex);
			FString TextureLoadPath = FString(FString("/Game/ItemTextures/TEX_ItemTexture_") +
				ItemTypeTag + FString("_") + ItemUnitTag + FString(".TEX_ItemTexture_") +
				ItemTypeTag + FString("_") + ItemUnitTag);
			return LoadObject<UTexture2D>(NULL, *TextureLoadPath, NULL, LOAD_None, NULL) != nullptr;
		}
		return false;
	}
protected:
	virtual TSharedPtr<SWidget> CreateDecoratorWidget(const FTextRunInfo& RunInfo, const FTextBlockStyle& TextStyle) const override
	{
		const FString* ItemTypeString = RunInfo.MetaData.Find(TEXT("itemtype"));
		const FString* ItemUnitString = RunInfo.MetaData.Find(TEXT("itemunit"));
		FString TextureLoadPath = FString(FString("/Game/ItemTextures/TEX_ItemTexture_") +
			*ItemTypeString + FString("_") + *ItemUnitString + FString(".TEX_ItemTexture_") +
			*ItemTypeString + FString("_") + *ItemUnitString);
		UTexture2D* Texture = LoadObject<UTexture2D>(NULL, *TextureLoadPath, NULL, LOAD_None, NULL);
		Decorator->_Brush = UWidgetBlueprintLibrary::MakeBrushFromTexture(Texture, 64, 64);
		TOptional<int32> Width;
		if (const FString* WidthString = RunInfo.MetaData.Find(TEXT("width")))
		{
			int32 WidthTemp;
			Width = FDefaultValueHelper::ParseInt(*WidthString, WidthTemp) ? WidthTemp : TOptional<int32>();
		}
		TOptional<int32> Height;
		if (const FString* HeightString = RunInfo.MetaData.Find(TEXT("height")))
		{
			int32 HeightTemp;
			Height = FDefaultValueHelper::ParseInt(*HeightString, HeightTemp) ? HeightTemp : TOptional<int32>();
		}
		EStretch::Type Stretch = EStretch::ScaleToFit;
		if (const FString* SstretchString = RunInfo.MetaData.Find(TEXT("stretch")))
		{
			static const UEnum* StretchEnum = StaticEnum<EStretch::Type>();
			int64 StretchValue = StretchEnum->GetValueByNameString(*SstretchString);
			if (StretchValue != INDEX_NONE)
			{
				Stretch = static_cast<EStretch::Type>(StretchValue);
			}
		}
		TOptional<int32> MarginTop;
		if (const FString* MarginTopString = RunInfo.MetaData.Find(TEXT("margin-top")))
		{
			int32 MarginTopTemp;
			MarginTop = FDefaultValueHelper::ParseInt(*MarginTopString, MarginTopTemp) ? MarginTopTemp : TOptional<int32>();
		}
		TOptional<int32> MarginBottom;
		if (const FString* MarginBottomString = RunInfo.MetaData.Find(TEXT("margin-bottom")))
		{
			int32 MarginBottomTemp;
			MarginBottom = FDefaultValueHelper::ParseInt(*MarginBottomString, MarginBottomTemp) ? MarginBottomTemp : TOptional<int32>();
		}
		TOptional<int32> MarginRight;
		if (const FString* MarginRightString = RunInfo.MetaData.Find(TEXT("margin-right")))
		{
			int32 MarginRightTemp;
			MarginRight = FDefaultValueHelper::ParseInt(*MarginRightString, MarginRightTemp) ? MarginRightTemp : TOptional<int32>();
		}
		TOptional<int32> MarginLeft;
		if (const FString* MarginLeftString = RunInfo.MetaData.Find(TEXT("margin-left")))
		{
			int32 MarginLeftTemp;
			MarginLeft = FDefaultValueHelper::ParseInt(*MarginLeftString, MarginLeftTemp) ? MarginLeftTemp : TOptional<int32>();
		}
		TOptional<float> PositionX;
		if (const FString* PositionXString = RunInfo.MetaData.Find(TEXT("x")))
		{
			float PositionXTemp;
			PositionX = FDefaultValueHelper::ParseFloat(*PositionXString, PositionXTemp) ? PositionXTemp : TOptional<float>();
		}
		TOptional<float> PositionY;
		if (const FString* PositionYString = RunInfo.MetaData.Find(TEXT("y")))
		{
			float PositionYTemp;
			PositionY = FDefaultValueHelper::ParseFloat(*PositionYString, PositionYTemp) ? PositionYTemp : TOptional<float>();
		}
		FCustomRichInlineImageArguments CustomRichInlineImageArguments = FCustomRichInlineImageArguments::FCustomRichInlineImageArguments(Width, Height, MarginTop, MarginBottom, MarginRight, MarginLeft, PositionX, PositionY);
		return SNew(SItemIconRichInlineImage, &Decorator->_Brush, TextStyle, Stretch, CustomRichInlineImageArguments);
	}
private:
	UItemIconRichTextBlockImageDecorator* Decorator;
};

#pragma endregion

#pragma endregion

#pragma region Item Icon Rich Text Block Image Decorator

UItemIconRichTextBlockImageDecorator::UItemIconRichTextBlockImageDecorator(
	const FObjectInitializer& ObjectInitializer) : URichTextBlockImageDecorator(ObjectInitializer) 
{
	_Brush = FSlateBrush();
}

TSharedPtr<ITextDecorator> UItemIconRichTextBlockImageDecorator::CreateDecorator(URichTextBlock* InOwner)
{
	return MakeShareable(new FItemIconRichInlineImage(InOwner, this));
}

#pragma endregion